//
//  ContentView.swift
//  ALD01a
//
//  Created by 富樫 豊彦 on 2024/03/05.
//

import SwiftUI
import Foundation
import CoreBluetooth

struct ContentView: View {
    @StateObject var btm = BTmanage()
    
    var body: some View {
        VStack {
            Text("Auto Lock Door")
                .font(.title)

            Label(btm.msgs, systemImage: btm.stat)
                .imageScale(.large)

            .padding()

            Button("Lock", systemImage: "lock") {
                btm.doorClose()
            }
            .frame(width: 100.0, height: 100.0)

            Button("UnLock", systemImage: "lock.open") {
                btm.doorOpen()
            }
            .frame(width: 100.0, height: 100.0)

        }

        .padding()
        Text("(c)2024 Toyohiko TOGASHI")
            .font(.caption2)
    }
}

// --- Bluetooth task ---

class BTmanage: NSObject, ObservableObject, CBCentralManagerDelegate, CBPeripheralDelegate {
    let sv = CBUUID(string: "1815") // GATT Automation IO service
    let cr = CBUUID(string: "2A56") // Digital I/O
    var cm: CBCentralManager!
    var dp: CBPeripheral!           // discoverd peripheral
    var dc: CBCharacteristic!       // discoverd characteristic
    @Published var msgs = ""
    @Published var stat = "lock.slash"

    override init(){
        super.init()
        cm = CBCentralManager(delegate: self, queue: nil, options: [CBCentralManagerOptionShowPowerAlertKey: true])
        print("BT init")
    }
    
    // --- CBCentralManagerDelegate ---
    
    func centralManagerDidUpdateState(_ m: CBCentralManager) {
        print("BT changed state \(m.state)")
        if m.state == .poweredOn {
            cm.scanForPeripherals(withServices: [sv], options: nil)
            msgs = "Scan start..."
        } else {
            msgs = "Environment error \(m.state.rawValue) ! "
        }
    }
    
    func centralManager(_ central: CBCentralManager, didDiscover p: CBPeripheral, advertisementData: [String : Any], rssi r: NSNumber) {
        print("BT didDiscover \(p) RSSI=\(r)")
        let m = p.name!
        msgs = m + " at " + r.stringValue + "[db]"
        if ((m.prefix(3) == "ALD") && (r.intValue > -80)) {
            print("Hit")
            if dp != p {
                dp = p
                cm.connect(p, options: nil)
            }
        }
    }
    
    func centralManager(_ central: CBCentralManager, didConnect p: CBPeripheral) {
        print("BT didConnect \(p)")
        msgs = "Connected " + p.name!
        cm.stopScan()
        p.delegate = self
        p.discoverServices([sv])    // Control to CBPeripheralDelegate
    }
    
    func centralManager(_ central: CBCentralManager, didFailToConnect p: CBPeripheral, error e: Error?) {
        print("BT didFailToConnect \(p) error=\(String(describing: e?.localizedDescription))")
        msgs = String(describing: e?.localizedDescription)
    }
    
    func centralManager(_ central: CBCentralManager, didDisconnectPeripheral p: CBPeripheral, error e: Error?) {
        print("BT didDisconnectPeripheral \(p) error=\(String(describing: e?.localizedDescription))")
        dp = nil
        dc = nil
        cm.scanForPeripherals(withServices: [sv], options: nil)
        msgs = "Searching..."
        stat = "lock.slash"
    }
    
    // --- CBPeripheralDelegate ---
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverServices e: Error?) {
        print("BT didDiscoverServices")
        if e == nil {
            for s in peripheral.services! {
                peripheral.discoverCharacteristics([cr], for: s)
                print("sv=\(s)")
            }
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral, didDiscoverCharacteristicsFor s: CBService, error e: Error?) {
        print("BT didDiscoverCharacteristicsFor")
        if e == nil {
            for c in s.characteristics! {
                print("char=\(c)")
                dc = c
                peripheral.setNotifyValue(true, for: c)
                self.doorOpen()     // default action
            }
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }

    func peripheral(_ peripheral: CBPeripheral, didUpdateValueFor c: CBCharacteristic, error e: Error?) {
        print("BT didUpdateValueFor")
        if e == nil {
            print("char=\(c)")
            let f = c.value?.withUnsafeBytes { $0.load( as: UInt8.self ) }
            if f == 0 {
                stat = "lock.open"
                print("Door Open")
            } else {
                stat = "lock"
                print("Door Close")
            }
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }
    
    func peripheral(_ peripheral: CBPeripheral, didUpdateNotificationStateFor c: CBCharacteristic, error e: Error?) {
        print("BT didUpdateNotificationStateFor")
        if e == nil {
            print("char=\(c)")
        } else {
            msgs = String(describing: e?.localizedDescription)
        }
    }
    
    func peripheralIsReady(toSendWriteWithoutResponse p: CBPeripheral) {
        print("BT toSendWriteWithoutResponse")
        print("char=\(p)")
    }
    
    
    // --- my procedure ---
    
    func doorOpen() {
        if (dp != nil) && (dc != nil) {
            let d = Data(bytes: [0], count: 1)
            dp.writeValue(d, for: dc, type: CBCharacteristicWriteType.withResponse)
        }
    }
    
    func doorClose() {
        if (dp != nil) && (dc != nil) {
            let d = Data(bytes: [1], count: 1)
            dp.writeValue(d, for: dc, type: CBCharacteristicWriteType.withResponse)
        }
    }
}

#Preview {
    ContentView()
}
